@@ -220,8 +220,8 @@ impl<
     /// again; same goes for the initial writing of a CSM message that is mandatory in CoAP over TCP.
     ///
     /// Any failure to write at response time, as well as protocol errors, are fatal and propagate out
-    /// as errors. In that case, the function must not be called on this socket again, but the socket
-    /// must be closed.
+    /// as errors. In that case, the socket gets closed, and the function must not be called on
+    /// this socket again.
     ///
     /// There is no successful return; whenever all pending requests have been processed, `WouldBlock`
     /// indicates that the function has done all it can do right now.
@@ -251,6 +251,27 @@ impl<
         handler: &mut impl coap_handler::Handler,
         fullbuf: &mut [u8; CONFIGURED_BUFLEN],
     ) -> Result<!, Error<ST::Error>>
+    where
+        ST: embedded_nal::TcpClientStack + embedded_nal_tcpextensions::TcpExactStack,
+    {
+        let result = self.poll_connection_with_buffer_nonclosing(stack, handler, fullbuf);
+        if matches!(result, Err(nb::Error::Other(_))) {
+            let sock = self.socket.take().expect("Socket removed prematurely");
+            // Trying once, can't do any more as this is now being dropped. At least the embedded-nal
+            // API doesn't expect this to WouldBlock anyway, so this just swallows additional errors.
+            let _ = stack.close(sock);
+        }
+        result
+    }
+
+    /// Workhorse for poll_connection_with_buffer (and thus poll_connection). The wrapper takes
+    /// care of closing the connection once an actual error occurs.
+    fn poll_connection_with_buffer_nonclosing(
+        &mut self,
+        stack: &mut ST,
+        handler: &mut impl coap_handler::Handler,
+        fullbuf: &mut [u8; CONFIGURED_BUFLEN],
+    ) -> Result<!, Error<ST::Error>>
     where
         // "Client" is not precisely it ... it really means "connected" here
         ST: embedded_nal::TcpClientStack + embedded_nal_tcpextensions::TcpExactStack,
@@ -539,18 +560,19 @@ impl<
 
     /// Send an Abort message and terminate the connection
     ///
-    /// Note that errors are not propagated out of this -- send failures are ignored (and no more
-    /// sends attempted), as are failures to close the connection.
+    /// Note that further errors are not propagated out of this -- send failures are ignored (and
+    /// no more sends attempted), as are failures to close the connection.
     ///
-    /// This closes the socket, and no further operations may be done on this state.
+    /// This does not close the connection, but the error propagating through the wrapper around
+    /// poll_connection_with_buffer_nonclosing will.
     ///
     /// # Returns
     ///
     /// an always-erring result to ease use as `return self.abort(error);`
     fn abort(&mut self, stack: &mut ST, error: Error<ST::Error>) -> Result<!, Error<ST::Error>> {
-        let mut socket = self
+        let socket = self
             .socket
-            .take()
+            .as_mut()
             .expect("Polled after poll returned an error");
 
         // It's a bit minimal, we could be nice and send text, but hey at least we tell that we
@@ -558,11 +580,7 @@ impl<
         //
         // Errors are discarded, we can't nb out of this, and rather return the root cause rather
         // than the follow-up.
-        let _ = stack.send_all(&mut socket, b"\x00\xe5"); // Abort
-
-        // Trying once, can't do any more as this is now being dropped. At least the embedded-nal
-        // API doesn't expect this to WouldBlock anyway, so this just swallows additional errors.
-        let _ = stack.close(socket);
+        let _ = stack.send_all(socket, b"\x00\xe5"); // Abort
 
         Err(nb::Error::Other(error))
     }
